#!/usr/bin/env python

try:
	import psyco
	psyco.full()
except ImportError:
	pass

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image, pygame.mixer
from pygame.locals import *

from levels import *

level=None

SCREEN_WIDTH = 480
SCREEN_HEIGHT = 448

MAP_SECTION_X=0
MAP_SECTION_Y=32
MAP_SECTION_WIDTH=SCREEN_WIDTH
MAP_SECTION_HEIGHT=176
LOWER_SECTION_X=0
LOWER_SECTION_Y=240
LOWER_SECTION_WIDTH=SCREEN_WIDTH
LOWER_SECTION_HEIGHT=160
TRACK_SECTION_X=0
TRACK_SECTION_Y=400
TRACK_SECTION_WIDTH=SCREEN_WIDTH
TRACK_SECTION_HEIGHT=16

DISTANT_HILLS_START=128
DISTANT_HILLS_HEIGHT=16

high_scores = [(123456,"CIW"),(12345,"VMW"),(1234,"DCW"),(123,"MDW"),(12,"JEW")]
player = []
current_player = 0

TITLE_PAUSE=2000
INSTRUCTIONS_PAUSE=3000
HIGH_SCORE_PAUSE=2000
TEXT_FLASH_RATE=200
SCROLL_SCALE=240

BLACK = (0.0, 0.0, 0.0)
RED = (1.0, 0.0, 0.0)
GREEN = (0.0, 1.0, 0.0)
BLUE = (0.0, 0.0, 1.0)
YELLOW = (1.0, 1.0, 0.0)
PURPLE = (1.0, 0.0, 1.0)
CYAN = (0.0, 1.0, 1.0)
WHITE = (1.0, 1.0, 1.0)

SKY_COLOUR_1=(0.0,0.0,0.5,1.0)
SKY_COLOUR_2=(0.5,0.5,1.0,1.0)

MAX_FORWARD_SPEED = 80
MAX_BACKWARD_SPEED = 60

SPRITE_INTERNAL_POSITION_SCALING=120
SCREEN_TO_MAP_RATIO=SCREEN_WIDTH/SPRITE_INTERNAL_POSITION_SCALING

SPRITE_PLAYER, SPRITE_TRAIN, SPRITE_FLYING, SPRITE_STATIC = range(4)
INIT_TITLE_SCREEN, TITLE_SCREEN, INSTRUCTIONS, DISPLAY_HIGH_SCORES, ENTER_HIGH_SCORE, GAME_STARTING, PLAYER_KILLED ,GAME_OVER, GAME_PLAYING, GAME_PAUSED, GAME_ESCAPE, PRESENT_TITLE = range(12)

METER_HEIGHT=7

WEAPON_LEFT = K_a
WEAPON_RIGHT = K_d
WEAPON_UP = K_w

UP=1
DOWN=2
AHEAD=4

junctions={
	118: (UP|AHEAD,      AHEAD),
	84 : (AHEAD,         AHEAD|DOWN),
	119: (AHEAD,         UP|AHEAD),
	89 : (AHEAD|DOWN,    AHEAD),
	121: (UP,            AHEAD),
	87 : (DOWN,          AHEAD),
	86 : (AHEAD,         DOWN),
	72 : (UP|AHEAD,      AHEAD|DOWN),
	92 : (UP|AHEAD,      DOWN),
	125: (AHEAD,         UP|AHEAD|DOWN),
	108: (DOWN,          UP|AHEAD),
	109: (UP,            AHEAD|DOWN),
	40 : (AHEAD|DOWN,    UP|AHEAD),
	116: (AHEAD,         UP),
	124: (UP|AHEAD|DOWN, AHEAD),
	93 : (AHEAD|DOWN,    UP)
	}

COAL_BLOCK=41
BUFFER_BLOCK=42
GO_BLOCK=55
STOP_BLOCK=56
OIL_BLOCK=57
WATER_BLOCK=58
TRACK_BLOCK=145

action_blocks=(COAL_BLOCK, BUFFER_BLOCK, STOP_BLOCK, OIL_BLOCK, WATER_BLOCK)

TITLE_MUSIC_FILENAME="rydeen.mid"
INGAME_MUSIC_FILENAME="rydeen.mid"
HIGH_SCORE_MUSIC_FILENAME="rydeen.mid"
MUSIC_VOLUME=0.1

pygame.init()
pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT), OPENGL|DOUBLEBUF)
glEnable(GL_TEXTURE_2D)


def load_textures(file_list):
	textures=glGenTextures(len(file_list))
	for texnum in range(len(textures)):
		textureSurface = pygame.image.load(file_list[texnum])
		textureData = pygame.image.tostring(textureSurface, "RGBA", 1)
		glBindTexture(GL_TEXTURE_2D, textures[texnum])
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textureSurface.get_width(), textureSurface.get_height(), 0, GL_RGBA,GL_UNSIGNED_BYTE, textureData)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
	return textures


def create_dlist_array(texnum, src_width, src_height, display_width=-1, display_height=-1, centre_origin=False, extra_first_block=False, auto_advance=True):
	global textures
	if display_width==-1:
		display_width=src_width
	if display_height==-1:
		display_height=src_height
	glBindTexture(GL_TEXTURE_2D, textures[texnum])
	TEXTURE_WIDTH=glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH)
	TEXTURE_HEIGHT=glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT)
	PIXEL_WIDTH=1.0/TEXTURE_WIDTH
	PIXEL_HEIGHT=1.0/TEXTURE_HEIGHT
	BLOCK_COUNT_Y=TEXTURE_HEIGHT//src_height
	BLOCK_COUNT_X=TEXTURE_WIDTH//src_width
	SRC_WIDTH=src_width
	SRC_HEIGHT=src_height
	DISPLAY_HEIGHT=display_height
	DISPLAY_WIDTH=display_width
	TEXTURE_BLOCK_WIDTH=PIXEL_WIDTH*src_width
	TEXTURE_BLOCK_HEIGHT=PIXEL_HEIGHT*src_height

	if centre_origin:
		vert=(-DISPLAY_WIDTH/2.0, DISPLAY_WIDTH/2.0, -DISPLAY_HEIGHT/2.0, DISPLAY_HEIGHT/2.0)
	else:
		vert=(0, DISPLAY_WIDTH, 0, DISPLAY_HEIGHT)

	base=glGenLists(BLOCK_COUNT_X*BLOCK_COUNT_Y)
	if extra_first_block:
		glNewList(base, GL_COMPILE)
		glTranslate(DISPLAY_WIDTH, 0, 0)
		glEndList()
		base += 1
	for y in range(BLOCK_COUNT_Y):
		for x in range(BLOCK_COUNT_X):
			glNewList(base+(y*BLOCK_COUNT_X)+x, GL_COMPILE)
			glBegin(GL_QUADS)
			glTexCoord2f((x*TEXTURE_BLOCK_WIDTH), 1.0-(y*TEXTURE_BLOCK_HEIGHT))
			glVertex3i(vert[0], vert[2], 0)
			glTexCoord2f((x+1)*TEXTURE_BLOCK_WIDTH, 1.0-(y*TEXTURE_BLOCK_HEIGHT))
			glVertex3i(vert[1], vert[2], 0)
			glTexCoord2f((x+1)*TEXTURE_BLOCK_WIDTH, 1.0-((y+1)*TEXTURE_BLOCK_HEIGHT))
			glVertex3i(vert[1], vert[3], 0)
			glTexCoord2f(x*TEXTURE_BLOCK_WIDTH, 1.0-((y+1)*TEXTURE_BLOCK_HEIGHT))
			glVertex3i(vert[0], vert[3], 0)
			glEnd()
			if auto_advance:
				glTranslate(DISPLAY_WIDTH, 0, 0)
			glEndList()
	if extra_first_block:
		base -= 1
	return base


def init_graphics():
	global textures, fontbase, upper_blocks, lower_strips, train_sprites, smoke_sprite, life_sprite, upper_sprites, panel_sprite, METER_TEXTURE_PIXEL_HEIGHT, steam_sprite, training_sprite, pyweek_sprite, chris_sprite, ground_sprite, weapon_sprite, GROUND_TEXTURE_PIXEL_HEIGHT, GROUND_TEXTURE_WIDTH, GROUND_TEXTURE_REPEAT_LENGTH, GROUND_TEXTURE_PIXEL_WIDTH, ground_base, scenery_base
	texture_list=("upper_map_blocks.png", "font.png", "smoke.png", "train_sprites.png", "smoke.png", "meters.png", "life_icon.png", "upper_sprites.png", "panel.png", "steam.png", "training.png", "pyweek.png", "chriswilson.png", "ground.png", "weapon.png")
	textures=load_textures(texture_list)
	glBindTexture(GL_TEXTURE_2D, textures[4])
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
	glBindTexture(GL_TEXTURE_2D, textures[14])
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
	glBindTexture(GL_TEXTURE_2D, textures[5])
	METER_TEXTURE_PIXEL_HEIGHT=1.0/glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT)

	glBindTexture(GL_TEXTURE_2D, textures[13])
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
	GROUND_TEXTURE_PIXEL_HEIGHT=1.0/glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT)
	GROUND_TEXTURE_PIXEL_WIDTH=1.0/glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH)
	GROUND_TEXTURE_WIDTH=glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH)
	GROUND_TEXTURE_REPEAT_LENGTH=LOWER_SECTION_WIDTH/float(GROUND_TEXTURE_WIDTH)

	ground_base=glGenLists(64)
	for i in range(64):
		glNewList(ground_base+i, GL_COMPILE)
		glBegin(GL_QUADS)
		glTexCoord(0,(160-i)*GROUND_TEXTURE_PIXEL_HEIGHT)
		glVertex(0,336+i)
		glTexCoord(2.0,(160-i)*GROUND_TEXTURE_PIXEL_HEIGHT)
		glVertex(1024,336+i)
		glTexCoord(2.0,(159-i)*GROUND_TEXTURE_PIXEL_HEIGHT)
		glVertex(1024,337+i)
		glTexCoord(0,(159-i)*GROUND_TEXTURE_PIXEL_HEIGHT)
		glVertex(0,337+i)
		glEnd()
		glEndList()
	
	scenery_base=glGenLists(3)
	for i in ((256,32,304,0),(224,64,272,1),(96,64,400,2)):
		glNewList(scenery_base+i[3], GL_COMPILE)
		glBegin(GL_QUADS)
		glTexCoord(0,(i[0])*GROUND_TEXTURE_PIXEL_HEIGHT)
		glVertex(0,i[2])
		glTexCoord(2.0,(i[0])*GROUND_TEXTURE_PIXEL_HEIGHT)
		glVertex(1024,i[2])
		glTexCoord(2.0,(i[0]-i[1])*GROUND_TEXTURE_PIXEL_HEIGHT)
		glVertex(1024,i[2]+i[1])
		glTexCoord(0,(i[0]-i[1])*GROUND_TEXTURE_PIXEL_HEIGHT)
		glVertex(0,i[2]+i[1])
		glEnd()
		glEndList()

	fontbase=create_dlist_array(1, 8, 8, 16, 16)
	upper_blocks=create_dlist_array(0,16,16,extra_first_block=True)
	train_sprites=create_dlist_array(3, 82, 56,centre_origin=True, auto_advance=False)
	smoke_sprite=create_dlist_array(4, 25, 26, centre_origin=True,auto_advance=False)
	life_sprite=create_dlist_array(6,48,32,auto_advance=False)
	upper_sprites=create_dlist_array(7,21,24,centre_origin=True)
	panel_sprite=create_dlist_array(8,480,32,auto_advance=False)
	steam_sprite=create_dlist_array(9,168,32,auto_advance=False)
	training_sprite=create_dlist_array(10,308,60,auto_advance=False)
	pyweek_sprite=create_dlist_array(11,128,32,auto_advance=False)
	chris_sprite=create_dlist_array(12,220,27,auto_advance=False)
	weapon_sprite=create_dlist_array(14,32,32,centre_origin=True, auto_advance=False)


def do_meter(xpos, ypos, meter_number, value):
	glBindTexture(GL_TEXTURE_2D,textures[5])
	glBegin(GL_QUADS)
	glTexCoord2f(0.0, METER_TEXTURE_PIXEL_HEIGHT*((8*meter_number)+METER_HEIGHT))
	glVertex3i(xpos, ypos+METER_HEIGHT, 0)
	glTexCoord2f(0.0, METER_TEXTURE_PIXEL_HEIGHT*(8*meter_number))
	glVertex3i(xpos, ypos, 0)
	glTexCoord2f(value/128.0, METER_TEXTURE_PIXEL_HEIGHT*(8*meter_number))
	glVertex3i(xpos+value, ypos, 0)
	glTexCoord2f(value/128.0, METER_TEXTURE_PIXEL_HEIGHT*((8*meter_number)+METER_HEIGHT))
	glVertex3i(xpos+value, ypos+METER_HEIGHT, 0)
	glEnd()
			

class map_sprite(object):
	def __init__(self, x=0, y=0, image=11, dy = 0, speed = 0, type=SPRITE_STATIC):
		self.x = x*SPRITE_INTERNAL_POSITION_SCALING
		self.y = y*SPRITE_INTERNAL_POSITION_SCALING
		self.dy = dy
		self.speed = speed
		self.image = image
		self.type = type
		self.rotation=0
		self.big_screen_x_pos=0
	def process(self):
		global scroll_position
		self.x+=self.speed
		self.y+=self.dy*self.speed

		if self.type==SPRITE_PLAYER:
			if self.x<(MAP_SECTION_WIDTH*SPRITE_INTERNAL_POSITION_SCALING)//2:
				scroll_position=0
			elif self.x>((512*16)*SPRITE_INTERNAL_POSITION_SCALING)-(MAP_SECTION_WIDTH*SPRITE_INTERNAL_POSITION_SCALING):
				scroll_position=((512*16)*SPRITE_INTERNAL_POSITION_SCALING)-(MAP_SECTION_WIDTH*SPRITE_INTERNAL_POSITION_SCALING)
			else:
				scroll_position=self.x-((MAP_SECTION_WIDTH*SPRITE_INTERNAL_POSITION_SCALING)//2)
				
		if self.type!=SPRITE_STATIC:
			self.map_x,self.map_x_offset=divmod((self.x//SPRITE_INTERNAL_POSITION_SCALING)+8,16)
			self.map_y,self.map_y_offset=divmod((self.y//SPRITE_INTERNAL_POSITION_SCALING)+8,16)
			self.current_map_block=level[self.map_y][self.map_x]
	
			if self.map_x_offset==8:
				if junctions.has_key(self.current_map_block):
					junction=junctions[self.current_map_block]
					if self.speed < 0:
						if (junction[0]&UP and keys[K_UP]) or junction[0]==UP:
							self.dy=1
						if (junction[0]&AHEAD and keys[K_LEFT]) or junction[0]==AHEAD:
							self.dy=0
						if (junction[0]&DOWN and keys[K_DOWN]) or junction[0]==DOWN:
							self.dy=-1
					else:
						if (junction[1]&UP and keys[K_UP]) or junction[1]==UP:
							self.dy=-1
						if (junction[1]&AHEAD and keys[K_RIGHT]) or junction[1]==AHEAD:
							self.dy=0
						if (junction[1]&DOWN and keys[K_DOWN]) or junction[1]==DOWN:
							self.dy=1

			if (self.dy==-1 and self.rotation>-45) or (self.dy==0 and self.rotation>0):
				self.rotation-=2
				
			if (self.dy==1 and self.rotation<45) or (self.dy==0 and self.rotation<0):
				self.rotation+=2
			if self.current_map_block in action_blocks:
				if self.current_map_block == OIL_BLOCK:
					level[self.map_y][self.map_x]=TRACK_BLOCK
					player[current_player].oil+=64
					if player[current_player].oil > 128:
						player[current_player].oil=128
				if self.current_map_block == WATER_BLOCK:
					level[self.map_y][self.map_x]=TRACK_BLOCK
					player[current_player].water+=64
					if player[current_player].water > 128:
						player[current_player].water=128
				if self.current_map_block == COAL_BLOCK:
					level[self.map_y][self.map_x]=TRACK_BLOCK
					player[current_player].coal+=64
					if player[current_player].coal > 128:
						player[current_player].coal=128
		self.big_screen_x_pos=-SCREEN_WIDTH + ((self.x*SCREEN_TO_MAP_RATIO)//SPRITE_INTERNAL_POSITION_SCALING-(SCREEN_WIDTH//2)) - ((scroll_position *SCREEN_TO_MAP_RATIO)//SPRITE_INTERNAL_POSITION_SCALING)
			
	def draw(self):
		glBindTexture(GL_TEXTURE_2D,textures[7])
		glEnable(GL_BLEND)
		glColor(1.0, 1.0, 1.0, 1.0)
		glPushMatrix()
		glTranslate(MAP_SECTION_X+8+(self.x-scroll_position)//SPRITE_INTERNAL_POSITION_SCALING, MAP_SECTION_Y+8+self.y//SPRITE_INTERNAL_POSITION_SCALING, 0)
		glRotate(self.rotation,0,0,1.0)
		glCallList(upper_sprites+self.image)
		glPopMatrix()
		

class player_class(object):
	def __init__(self):
		self.score = 0
		self.lives = 3
		self.level = 1
		self.hits = 0
		self.oil = 128
		self.water = 80
		self.coal = 48
		self.pressure = 64
		self.round = 1
		self.weapon_angle=0
		self.weapon_dest_angle=0

player = [player_class(), player_class()]
current_player = 0


def do_print(xpos, ypos, text, colour = WHITE, shadow = False, alpha = 1.0):
	glBindTexture(GL_TEXTURE_2D, textures[1])
	if shadow:
		glPushMatrix()
		glTranslate(xpos + 2, ypos + 2,0)
		glColor4f(0.0, 0.0, 0.0, alpha)
		glCallLists(map(lambda x,y:(ord(x)-32)+fontbase, text, [fontbase] * len(text)))
		glPopMatrix()
	glPushMatrix()
	glTranslate(xpos, ypos, 0)
	glColor4f(colour[0], colour[1], colour[2], alpha)
	glCallLists(map(lambda x,y:(ord(x) - 32)+fontbase, text, [fontbase] * len(text)))
	glPopMatrix()


def display_map(xpos, ypos, width, height, map_x, map_y, map_base, block_dlist_base, block_texture, block_width, block_height):
	glBindTexture(GL_TEXTURE_2D, textures[block_texture])
	glScissor(xpos, SCREEN_HEIGHT-(ypos+height), width, height)
	glEnable(GL_SCISSOR_TEST)
	glColor3f(1.0,1.0,1.0)
	block_count_x=(width/block_width)+1
	if map_x%block_width:
		block_count_x+=1
	block_count_y=(height/block_height)
	if block_count_y==1:
		block_temp=map_base[map_x/block_width:(map_x/block_width)+block_count_x]
		glPushMatrix()
		glTranslate(xpos-(map_x%block_width), ypos ,0)
		glCallLists(map(lambda x,y:(x+y), block_temp, [block_dlist_base]*len(block_temp)))
		glPopMatrix()
	else:
		if map_y%block_height:
			block_count_y+=1
		for y in range(block_count_y):
			block_temp=map_base[(map_y/block_height)+y][map_x/block_width:(map_x/block_width)+block_count_x]
			glPushMatrix()
			glTranslate(xpos-(map_x%block_width), ypos-(map_y%block_height)+(y*block_height), 0)
			glCallLists(map(lambda x,y:(x+y), block_temp, [block_dlist_base]*len(block_temp)))
			glPopMatrix()


def do_sprite(xpos, ypos, texnum, dlist, alpha=1.0, rotation=0):
	glEnable(GL_BLEND)
	glColor(1.0, 1.0, 1.0, alpha)
	glPushMatrix()
	glBindTexture(GL_TEXTURE_2D, textures[texnum])
	glTranslate(xpos, ypos, 0)
	glRotate(rotation,0.0,0.0,1.0)
	glCallList(dlist)
	glPopMatrix()


def play_music(music):
	if music == None:
		pygame.mixer.music.stop()
	else:
		pygame.mixer.music.load(music)
		pygame.mixer.music.set_volume(MUSIC_VOLUME)
		pygame.mixer.music.play(-1)


def main():
	global textures, fontbase, scroll_speed, smoke_count, keys, level, player, player_upper_sprite

	init_graphics()

	mode=INIT_TITLE_SCREEN
#	mode=GAME_STARTING

	level_scroll_pos = 0
	scroll_speed=0
	global_count=0
	ground_scroll_test=0
		
	while 1:

		global_count+=1
		keys=pygame.key.get_pressed()
		if mode == INIT_TITLE_SCREEN:
			glClearColor(0.0157, 0.0078, 0.2667, 1.0)
			glLoadIdentity()
			glOrtho(0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, -5, 100)
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
			glEnable(GL_BLEND)
			play_music(TITLE_MUSIC_FILENAME)
			count=800
			count2=0
			count3=0
			toggle=1
			mode=PRESENT_TITLE
		if mode == PRESENT_TITLE:
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			if count>0:
				count-=2
			elif count2<800:
				count2+=1
			elif count3 < 1.0:
				count3+=0.002
			else:
				count
				mode=TITLE_SCREEN
			do_sprite(89-count,32,9,steam_sprite)
			do_sprite(86+count,64,10,training_sprite)
			if count2>400:
				do_sprite(112,112,12,chris_sprite)
			do_sprite(176,384,11,pyweek_sprite,alpha=count3)
			pygame.display.flip()
		if mode == TITLE_SCREEN:
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			event = pygame.event.poll()
			if event.type == QUIT or keys[K_ESCAPE]:
				break
			if keys[K_F1]:
				num_players=1
				mode=GAME_STARTING
			if keys[K_F2]:
				num_players=2
				mode=GAME_STARTING
			do_sprite(89,32,9,steam_sprite)
			do_sprite(86,64,10,training_sprite)
			do_sprite(112,112,12,chris_sprite)
			do_sprite(176,384,11,pyweek_sprite)
			do_print(144, 224, "PLEASE PRESS", YELLOW, True)
			if (global_count%TEXT_FLASH_RATE*2)<TEXT_FLASH_RATE:
				do_print(88, 256, "1 OR 2 PLAYER START", WHITE, True)
			count+=1
			if count>=TITLE_PAUSE:
				count=0
				toggle=1-toggle
				if toggle:
					mode=INSTRUCTIONS
				else:
					mode=DISPLAY_HIGH_SCORES
			pygame.display.flip()
		elif mode == INSTRUCTIONS:
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			event = pygame.event.poll()
			if event.type == QUIT or keys[K_ESCAPE]:
				break
			if keys[K_F1]:
				num_players=1
				mode=GAME_STARTING
			if keys[K_F2]:
				num_players=2
				mode=GAME_STARTING
			do_sprite(89,32,9,steam_sprite)
			do_sprite(86,64,10,training_sprite)
			do_sprite(112,112,12,chris_sprite)
			do_sprite(176,384,11,pyweek_sprite)

			count+=1
			if count>=INSTRUCTIONS_PAUSE*2:
				count=0
				mode=TITLE_SCREEN
			elif count>INSTRUCTIONS_PAUSE:
				do_print(56, 176, "THIS GAME IS INCOMPLETE", RED, True)

				do_print(0, 208, "CURRENTLY ABOUT THE ONLY THING", YELLOW, True)
				do_print(8, 224, "YOU ARE ABLE TO DO, IS POOTLE", YELLOW, True)

				do_print(56, 240, "AROUND AND COLLECT OIL.", YELLOW, True)

				do_print(24, 272, "ONCE IT'S FINISHED, A FINAL", YELLOW, True)
				do_print(0, 288, "VERSION WILL BE AVAILABLE FROM", YELLOW, True)
				do_print(136, 314, "www.ciw42.net", YELLOW, True)
			else:
				do_print(154, 160, "HOW TO PLAY", CYAN, True)

				do_print(48, 192, "LEFT & RIGHT CURSOR KEYS")
				do_print(24, 208, "CHANGE YOUR SPEED/DIRECTION")

				do_print(72, 240, "UP & DOWN CURSOR KEYS")
				do_print(32, 256, "ALLOW YOU TO CHANGE TRACKS")

				do_print(56, 288, "THE \"A\", \"W\" & \"D\" KEYS")
				do_print(16, 304, "ALLOW YOU TO AIM YOUR WEAPON")
				do_print(8, 320, "(ALTHOUGH YOU CAN'T FIRE IT!)")
				
			pygame.display.flip()
		elif mode == DISPLAY_HIGH_SCORES:
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			event = pygame.event.poll()
			if event.type == QUIT or keys[K_ESCAPE]:
				break
			if keys[K_F1]:
				num_players=1
				mode=GAME_STARTING
			if keys[K_F2]:
				num_players=2
				mode=GAME_STARTING
			do_sprite(89,32,9,steam_sprite)
			do_sprite(86,64,10,training_sprite)
			do_sprite(112,112,12,chris_sprite)
			do_sprite(176,384,11,pyweek_sprite)

			do_print(64, 164, "TODAY'S HIGHEST SCORES", YELLOW, True)
			for i in range(5):
				do_print(112, 208+(i*32), "%1d. %7d - %3s"%(i+1,high_scores[i][0],high_scores[i][1]), WHITE, True)

			count+=1
			if count>=HIGH_SCORE_PAUSE:
				count=0
				mode=TITLE_SCREEN
			pygame.display.flip()
		elif mode == ENTER_HIGH_SCORE:
			pass
		elif mode == GAME_STARTING:
			play_music(INGAME_MUSIC_FILENAME)
			scroll_speed = 0
			mapx = 0
			mapy = 0
			current_level=0
			player = [player_class(), player_class()]
			current_player = 0
			level=level_data[current_level][:]
			player_upper_sprite=map_sprite(240,80,10,0,0,type=SPRITE_PLAYER)
			mode=GAME_PLAYING
		elif mode == PLAYER_KILLED:
			pass
		elif mode == GAME_OVER:
			pass
		elif mode == GAME_PAUSED:
			pass
		elif mode == GAME_ESCAPE:
			pass
		elif mode == GAME_PLAYING:
			event = pygame.event.poll()
			keys = pygame.key.get_pressed()
			if event.type == QUIT or keys[K_ESCAPE]:
				break
			keys = pygame.key.get_pressed()
			if not global_count % 5:
				if keys[K_LEFT] and player_upper_sprite.speed > -MAX_BACKWARD_SPEED:
					player_upper_sprite.speed -= 1
				if keys[K_RIGHT] and player_upper_sprite.speed < MAX_FORWARD_SPEED:
					player_upper_sprite.speed += 1
				if keys[WEAPON_LEFT]:
					player[current_player].weapon_dest_angle=-90
					if player[current_player].weapon_angle==player[current_player].weapon_dest_angle:
						pass
				elif keys[WEAPON_UP]:
					player[current_player].weapon_dest_angle=0
					if player[current_player].weapon_angle==player[current_player].weapon_dest_angle:
						pass
				elif keys[WEAPON_RIGHT]:
					player[current_player].weapon_dest_angle=90
					if player[current_player].weapon_angle==player[current_player].weapon_dest_angle:
						pass
			if player[current_player].weapon_angle < player[current_player].weapon_dest_angle:
				player[current_player].weapon_angle+=1
			elif player[current_player].weapon_angle > player[current_player].weapon_dest_angle:
				player[current_player].weapon_angle-=1

			glClearColor(0.0157, 0.0078, 0.2667, 1.0)
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			glLoadIdentity()
	
			glOrtho(0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, -5, 100)
	
			glDisable(GL_DEPTH_TEST)
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
	
			player_upper_sprite.process()

			display_map(MAP_SECTION_X, MAP_SECTION_Y, MAP_SECTION_WIDTH, MAP_SECTION_HEIGHT, scroll_position/SPRITE_INTERNAL_POSITION_SCALING, 0, level, upper_blocks, 0, 16, 16)

			player_upper_sprite.draw()

			glScissor(LOWER_SECTION_X,SCREEN_HEIGHT-(LOWER_SECTION_HEIGHT+LOWER_SECTION_Y),LOWER_SECTION_WIDTH,LOWER_SECTION_HEIGHT)
			glDisable(GL_TEXTURE_2D)
			glBegin(GL_QUADS)
			glColor(SKY_COLOUR_1)
			glVertex(0,240,0)
			glVertex(480,240,0)
			glColor(SKY_COLOUR_2)
			glVertex(480,368,0)
			glVertex(0,368,0)
			glEnd()
			glEnable(GL_TEXTURE_2D)

			glEnable(GL_BLEND)
			glColor(1.0, 1.0, 1.0, 1.0)
			glBindTexture(GL_TEXTURE_2D, textures[13])

			x_scroll=0*GROUND_TEXTURE_PIXEL_WIDTH

			ground_scroll=((scroll_position*2)//SPRITE_INTERNAL_POSITION_SCALING)
			
			glPushMatrix()
			glTranslate(-((ground_scroll*(0.40+((0.5/64))))%512),0,0)
			glCallList(scenery_base)
			glPopMatrix()

			glPushMatrix()
			glTranslate(-((ground_scroll*(0.475+((0.5/64))))%512),0,0)
			glCallList(scenery_base+1)
			glPopMatrix()

			for i in range(64):
				glPushMatrix()
				glTranslate(-((ground_scroll*(0.5+(i*(0.5/64))))%512),0,0)
				glCallList(ground_base+i)
				glPopMatrix()

			glDisable(GL_SCISSOR_TEST)
			glPushMatrix()
			glTranslate(-((ground_scroll*(1.5+((0.5/64))))%512),0,0)
			glCallList(scenery_base+2)
			glPopMatrix()

			do_sprite(player_upper_sprite.big_screen_x_pos,372,3,train_sprites+(((player_upper_sprite.x//SPRITE_INTERNAL_POSITION_SCALING)//6)%4))
			do_sprite(player_upper_sprite.big_screen_x_pos-30,363,14,weapon_sprite,rotation=player[current_player].weapon_angle)
			
			if not global_count%64 and player[current_player].oil > 0:
				player[current_player].oil-=1
			if not global_count%((MAX_FORWARD_SPEED-abs(scroll_speed))//8):
				player[0].score+=1

			glBindTexture(GL_TEXTURE_2D, textures[4])
			for i in range(5+abs(player_upper_sprite.speed//8)):
				smoke_count2 = ((global_count%512)+(i * (512 / (5+abs(player_upper_sprite.speed//8)))))%512
				glColor(1.0, 1.0, 1.0, 0.8 - ((0.8 / 384) * smoke_count2))
				glPushMatrix()
				glBindTexture(GL_TEXTURE_2D, textures[4])
				glTranslate(player_upper_sprite.big_screen_x_pos+8 - ((smoke_count2 * (float(player_upper_sprite.speed) / MAX_FORWARD_SPEED))/2), 340 - (smoke_count2 / 48),0)
				glRotate(smoke_count2 / 3, 0.0, 0.0, 1.0)
				glScale(0.25 + (4.0 / 512) * smoke_count2, 0.25 + (4.0 / 512) * smoke_count2, 1.0)
				glCallList(smoke_sprite)
				glPopMatrix()
				glPushMatrix()
				glBindTexture(GL_TEXTURE_2D, textures[4])
				glTranslate(player_upper_sprite.big_screen_x_pos+6 - (smoke_count2 * ((float(player_upper_sprite.speed) / MAX_FORWARD_SPEED))/2), 342 - (smoke_count2 / 48),0)
				glRotate(-smoke_count2 / 3, 0.0, 0.0, 1.0)
				glScale(0.25 + (4.0 / 512) * smoke_count2, 0.25 + (4.0 / 512) * smoke_count2, 1.0)
				glCallList(smoke_sprite)
				glPopMatrix()
			
			glDisable(GL_SCISSOR_TEST)
			do_print(32, 0, "PLYR.1   HI-SCORE   PLYR.2")
			do_print(32, 16, "%6d    %6d    %6d" % (player[0].score, high_scores[0][0], player[1].score))
			do_print(284, 432, "PYWEEK 2006")

			for i in range(player[current_player].lives-1):
				do_sprite(32+(i*50),414,6,life_sprite)

			do_meter(16,213,0,128)
			do_meter(16,213,3,player[current_player].oil)
			do_meter(16,228,0,128)
			do_meter(16,228,2,player[current_player].water)
			do_meter(336,213,0,128)
			do_meter(336,213,3,player[current_player].coal)
			do_meter(336,228,0,128)
			do_meter(336,228,2,player[current_player].pressure)
			do_sprite(0,208,8,panel_sprite)

			glFlush()
			pygame.display.flip()

	play_music(None)

if __name__ == '__main__':
	main()
